//+------------------------------------------------------------------+
//|                                                         OBV2.mq4 |
//+------------------------------------------------------------------+
#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  DodgerBlue
#property indicator_width1  2

//
//
//
//
//

extern string TimeFrame                     = "Current time frame";
extern int    obvPrice                      = 0;
extern int    LinearRegressionLength        = 14;
extern double LinearRegressionChannelWidth  = 2.0;
extern string IndicatorUniqueID             = "obv slope divergence";
extern color  ChartLineColor                = Gold;
extern int    ChartLineMiddleStyle          = STYLE_DOT;
extern int    ChartLineStyle                = STYLE_SOLID;
extern color  ObvLineColor                  = Gold;
extern int    ObvLineMiddleStyle            = STYLE_DOT;
extern int    ObvLineStyle                  = STYLE_SOLID;
extern bool   Interpolate                   = false;

//
//
//
//
//

double obv[];
string shortName;
string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;


//
//
//
//
//

int init()
{

   SetIndexBuffer(0,obv);
   
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame=="calculateValue");
      if (calculateValue)
      {
         int s = StringFind(IndicatorUniqueID,":",0);
               shortName = IndicatorUniqueID;
               IndicatorUniqueID = StringSubstr(IndicatorUniqueID,0,s);
               return(0);
      }            
      timeFrame = stringToTimeFrame(TimeFrame);
      
           
      shortName = IndicatorUniqueID+":  "+timeFrameToString(timeFrame)+" - obv ("+LinearRegressionLength+")";

   IndicatorShortName(shortName);
return(0);
}

//
//
//
//
//

int deinit()
{
   for (int i=ObjectsTotal(); i>=0; i--)
   {
      string name = ObjectName(i); if (StringFind(name,IndicatorUniqueID)==0) ObjectDelete(name);
   }
return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
          if (returnBars) { obv[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   if (calculateValue || timeFrame==Period())
   {    
     for (i=limit; i>=0; i--)
     {
   
        if (i==(Bars-1))                                                                     { obv[i] = Volume[i]; }
        else  { if((High[i] == Low[i]) || (Open[i] == Close[i]) || (Close[i] == Close[i+1])) { obv[i] = obv[i+1];  }
      
        else  { if (Close[i] > Open[i])   
                     obv[i] = obv[i+1] + (Volume[i] * (Close[i] - Open[i]) / (High[i] - Low[i]));
                else obv[i] = obv[i+1] - (Volume[i] * (Open[i] - Close[i]) / (High[i] - Low[i])); } }
        fillLrArray(i,0,obv[i]);
        fillLrArray(i,1,getPrice(obvPrice,i));            
     }
   
     //
     //
     //
     //
     //
   
     double obvError; double obvSlope; double lrobv = iLrValue(obv[0],              LinearRegressionLength,obvSlope,obvError,0,0);
     double prcError; double prcSlope; double lrPrc = iLrValue(getPrice(obvPrice,0),LinearRegressionLength,prcSlope,prcError,0,1);
     int window = WindowFind(shortName);
     createLine(window,lrobv,lrobv-(LinearRegressionLength-1.0)*obvSlope,"obvLine",ObvLineColor  ,ObvLineStyle  ,ObvLineMiddleStyle  ,obvError*LinearRegressionChannelWidth);
     createLine(0     ,lrPrc,lrPrc-(LinearRegressionLength-1.0)*prcSlope,"prcLine",ChartLineColor,ChartLineStyle,ChartLineMiddleStyle,prcError*LinearRegressionChannelWidth);
   
     return(0);
     }
     
     //
     //
     //
     //
     //
     
     limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
     for (i=limit; i>=0; i--)
     {
        int y = iBarShift(NULL,timeFrame,Time[i]);
           obv[i]      = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",obvPrice,LinearRegressionLength,LinearRegressionChannelWidth,shortName,ChartLineColor,ChartLineMiddleStyle,ChartLineStyle,ObvLineColor,ObvLineMiddleStyle,ObvLineStyle,0,y);
         
           //
           //
           //
           //
           //
        
           if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

           //
           //
           //
           //
           //

           datetime time = iTime(NULL,timeFrame,y);
              for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;
              for(int x = 1; x < n; x++) 
              {
              
                 obv[i+x] = obv[i] + (obv[i+n] - obv[i])* x/n;
              }
      
   }
return(0);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//
//

double getPrice(int type, int i)
{
   switch (type)
   {
      case 7:     return((Open[i]+Close[i])/2.0);
      case 8:     return((Open[i]+High[i]+Low[i]+Close[i])/4.0);
      default :   return(iMA(NULL,0,1,0,MODE_SMA,type,i));
   }      
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void createLine(int window, double price1, double price2, string addName, color theColor, int theStyle, int theMiddleStyle, double error)
{
   string name = IndicatorUniqueID+addName;
      if (ObjectFind(name)==-1)
           ObjectCreate(name,OBJ_TREND,window,0,0,0,0);
              ObjectSet(name,OBJPROP_PRICE1,price1);
              ObjectSet(name,OBJPROP_PRICE2,price2);
              ObjectSet(name,OBJPROP_TIME1,Time[0]);
              ObjectSet(name,OBJPROP_TIME2,Time[LinearRegressionLength-1]);
              ObjectSet(name,OBJPROP_RAY,false);
              ObjectSet(name,OBJPROP_COLOR,theColor);
              ObjectSet(name,OBJPROP_STYLE,theMiddleStyle);
                  if (error<=0) return;
      name = IndicatorUniqueID+addName+"up";
      if (ObjectFind(name)==-1)
           ObjectCreate(name,OBJ_TREND,window,0,0,0,0);
              ObjectSet(name,OBJPROP_PRICE1,price1+error);
              ObjectSet(name,OBJPROP_PRICE2,price2+error);
              ObjectSet(name,OBJPROP_TIME1,Time[0]);
              ObjectSet(name,OBJPROP_TIME2,Time[LinearRegressionLength-1]);
              ObjectSet(name,OBJPROP_RAY,false);
              ObjectSet(name,OBJPROP_COLOR,theColor);
              ObjectSet(name,OBJPROP_STYLE,theStyle);
      name = IndicatorUniqueID+addName+"down";
      if (ObjectFind(name)==-1)
           ObjectCreate(name,OBJ_TREND,window,0,0,0,0);
              ObjectSet(name,OBJPROP_PRICE1,price1-error);
              ObjectSet(name,OBJPROP_PRICE2,price2-error);
              ObjectSet(name,OBJPROP_TIME1,Time[0]);
              ObjectSet(name,OBJPROP_TIME2,Time[LinearRegressionLength-1]);
              ObjectSet(name,OBJPROP_RAY,false);
              ObjectSet(name,OBJPROP_COLOR,theColor);
              ObjectSet(name,OBJPROP_STYLE,theStyle);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double workLr[][2];
void fillLrArray(int i, int instanceNo, double value)
{
   if (ArrayRange(workLr,0)!=Bars) ArrayResize(workLr,Bars); i = Bars-i-1; workLr[i][instanceNo] = value;
}
double iLrValue(double value, int period, double& slope, double& error, int r, int instanceNo=0)
{
   if (ArrayRange(workLr,0)!=Bars) ArrayResize(workLr,Bars); r = Bars-r-1; workLr[r][instanceNo] = value;
   if (r<period || period<2) return(value);

   //
   //
   //
   //
   //

      double sumx=0, sumxx=0, sumxy=0, sumy=0, sumyy=0;
         for (int k=0; k<period; k++)
         {
            double price = workLr[r-k][instanceNo];
                   sumx  += k;
                   sumxx += k*k;
                   sumxy += k*price;
                   sumy  +=   price;
                   sumyy +=   price*price;
         }
         slope = (period*sumxy-sumx*sumy)/(sumx*sumx-period*sumxx);
         error = MathSqrt((period*sumyy-sumy*sumy-slope*slope*(period*sumxx-sumx*sumx))/(period*(period-2)));

   //
   //
   //
   //
   //
         
   return((sumy + slope*sumx)/period);
}

//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

